import argparse
import time
from anyscale import AnyscaleSDK
import os

import ray

ray.init(address="auto")

parser = argparse.ArgumentParser()
parser.add_argument("max_time_s", type=int, help="Wait for this number of seconds")
parser.add_argument(
    "num_nodes", type=int, nargs="?", help="Wait for this number of nodes (includes head)", default=-1,
)



args = parser.parse_args()

curr_nodes = 0
start = time.time()
next_feedback = start
max_time = start + args.max_time_s

cluster_id = os.environ.get("ANYSCALE_CLUSTER_ID") or os.environ.get("ANYSCALE_SESSION_ID")

sdk = None

if args.num_nodes == -1:
    if not cluster_id:
        raise RuntimeError("Could not find cluster id and num_nodes argument not provided. "
        "Cannot determine number of nodes needed to wait for. Aborting now...")
    print("num_nodes argument not provided. Will try to detect min nodes requested in compute config.")
    sdk = AnyscaleSDK()
    cluster = sdk.get_cluster(cluster_id).result
    compute_config = sdk.get_cluster_compute(cluster.cluster_compute_id).result.config
    # Calculate number of nodes to wait for: 1 for head node + sum of all worker nodes
    worker_node_types = compute_config.worker_node_types or []
    num_nodes = 1 + sum(worker_node.min_workers or 0 for worker_node in worker_node_types)
    print(f"Detected a total of {num_nodes} minimum nodes.")
else:
    num_nodes = args.num_nodes

if not cluster_id:
    print("Warning: Could not find cluster id... "
    "This script will fail but will not attempt to terminate the cluster for you. ")

while not curr_nodes >= num_nodes:
    now = time.time()

    if now >= max_time:
        print(
            f"Maximum wait time reached, but only "
            f"{curr_nodes}/{num_nodes} nodes came up. Aborting."
        )
        if not sdk:
            sdk = AnyscaleSDK()
        if cluster_id:
            sdk.terminate_cluster(cluster_id, {})
            time.sleep(5)
        else:
            raise RuntimeError("Could not find cluster id... Terminating app to let autosuspend kick in.")


    if now >= next_feedback:
        passed = now - start
        print(
            f"Waiting for more nodes to come up: "
            f"{curr_nodes}/{num_nodes} "
            f"({passed:.0f} seconds passed)"
        )
        next_feedback = now + 10

    time.sleep(5)
    curr_nodes = sum(1 for node in ray.nodes() if node["Alive"])

passed = time.time() - start
print(
    f"Cluster is up: {curr_nodes}/{num_nodes} nodes online after "
    f"{passed:.0f} seconds"
)
